"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var sp_core_library_1 = require("@microsoft/sp-core-library");
var sp_diagnostics_1 = require("@microsoft/sp-diagnostics");
var MSGraphClient_1 = tslib_1.__importDefault(require("./MSGraphClient"));
/**
 * Returns a preinitialized version of the MSGraphClient.
 * For more information: {@link https://docs.microsoft.com/en-us/sharepoint/dev/spfx/use-msgraph}
 *
 * @public
 */
var MSGraphClientFactory = /** @class */ (function () {
    /**
     * @param serviceScope - Provides services for the MSGraphClient to consume.
     *
     * @internal
     */
    function MSGraphClientFactory(serviceScope) {
        this._serviceScope = serviceScope;
    }
    MSGraphClientFactory.prototype.getClient = function (version, options) {
        var _this = this;
        if (!version || version === '1') {
            // returns a v1 version of MSGraphClient
            /* eslint-disable-next-line @typescript-eslint/no-explicit-any */
            if (!MSGraphClient_1.default._instance) {
                return Promise.resolve().then(function () { return tslib_1.__importStar(require(
                /* webpackChunkName: 'sp-http-msgraphclient' */ '@microsoft/microsoft-graph-clientv1/lib/graph-js-sdk-web')); }).then(function () {
                    return new MSGraphClient_1.default(_this._serviceScope);
                });
            }
            else {
                return Promise.resolve(new MSGraphClient_1.default(this._serviceScope));
            }
        }
        else if (version === '3' &&
            !sp_core_library_1._SPKillSwitch.isActivated('24401914-15d6-4c3c-aeea-4aace46d5586' /* '01/06/2022', 'use new MSGraphVersion' */)) {
            var qosMonitor_1 = new sp_diagnostics_1._QosMonitor('MSGraphClientFactory.GetClientV3');
            // returns a v3 version of MSGraphClient
            // Should only import once no matter how many times it is called
            return Promise.resolve().then(function () { return tslib_1.__importStar(require(/* webpackChunkName: 'sp-http-msgraphclientv3' */ './MSGraphClientV3')); }).then(function (msGraphClientV3) {
                // We create a new object every time rather than referencing an existing object in case the service scope changes
                qosMonitor_1.writeSuccess({ version: 3, options: !!options });
                return new msGraphClientV3.default(_this._serviceScope, options, undefined);
            })
                .catch(function (error) {
                qosMonitor_1.writeUnexpectedFailure('ImportMSGraphV3ChunkFailed', error);
                throw error;
            });
        }
        else {
            var errorMessage = 'Not yet implemented.';
            throw Error(errorMessage);
        }
    };
    /**
     * Returns an instance of the MSGraphClient that communicates with the current tenant's configurable Service Pricipal.
     * Allows to specify custom middleware options.
     *
     * @param version - The version of the Graph API to use. 3.
     * @param options - Client options for the Graph API.
     */
    MSGraphClientFactory.prototype.getClientWithMiddleware = function (version, options) {
        var _this = this;
        var qosMonitor = new sp_diagnostics_1._QosMonitor('MSGraphClientFactory.GetClientV3WithMiddleware');
        // returns a v3 version of MSGraphClient
        // Should only import once no matter how many times it is called
        return Promise.resolve().then(function () { return tslib_1.__importStar(require(/* webpackChunkName: 'sp-http-msgraphclientv3' */ './MSGraphClientV3')); }).then(function (msGraphClientV3) {
            // We create a new object every time rather than referencing an existing object in case the service scope changes
            qosMonitor.writeSuccess({ version: 3 });
            return new msGraphClientV3.default(_this._serviceScope, undefined, options);
        })
            .catch(function (error) {
            qosMonitor.writeUnexpectedFailure('ImportMSGraphV3ChunkFailed', error);
            throw error;
        });
    };
    /**
     * Returns a v3 version of MSGraphClient with the 1P App Principal
     *
     * @internal
     *
     */
    MSGraphClientFactory.prototype.getClientInternal = function (version, options) {
        var _this = this;
        var qosMonitor = new sp_diagnostics_1._QosMonitor('MSGraphClientFactory.GetInternalClientV3');
        // Should only import once no matter how many times it is called
        return Promise.resolve().then(function () { return tslib_1.__importStar(require(/* webpackChunkName: 'sp-http-msgraphclientv3' */ './MSGraphClientV3')); }).then(function (msGraphClientV3) {
            // We create a new object every time rather than referencing an existing object in case the service scope changes
            qosMonitor.writeSuccess({ version: 3, options: !!options });
            return new msGraphClientV3.default(_this._serviceScope, options, undefined, true);
        })
            .catch(function (error) {
            qosMonitor.writeUnexpectedFailure('ImportMSGraphV3ChunkFailed', error);
            throw error;
        });
    };
    /**
     * The service key for MSGraphClientFactory.
     */
    MSGraphClientFactory.serviceKey = sp_core_library_1.ServiceKey.create('sp-http:MSGraphClientFactory', MSGraphClientFactory);
    return MSGraphClientFactory;
}());
exports.default = MSGraphClientFactory;
//# sourceMappingURL=MSGraphClientFactory.js.map